@@ -7,16 +7,22 @@ use Protocol::WebSocket::Handshake::Server;
 use Try::Tiny;
 use AnyEvent::WebSocket::Connection;
 
-our $VERSION = "0.05";
+our $VERSION = "0.06";
 
 sub new {
     my ($class, %args) = @_;
-    my $validator = $args{validator} || sub {};
-    if(ref($validator) ne "CODE") {
+    my $validator = $args{validator};
+    if(defined($validator) && ref($validator) ne "CODE") {
         croak "validator parameter must be a code-ref";
     }
+    my $handshake = defined($args{handshake}) ? $args{handshake}
+        : defined($validator) ? sub { my ($req, $res) = @_; return ($res, $validator->($req)); }
+        : sub { $_[1] };
+    if(ref($handshake) ne "CODE") {
+        croak "handshake parameter must be a code-ref";
+    }
     my $self = bless {
-        validator => $validator
+        handshake => $handshake,
     }, $class;
     return $self;
 }
@@ -34,7 +40,7 @@ sub _create_on_error {
 }
 
 sub _do_handshake {
-    my ($cv_connection, $fh, $handshake, $validator) = @_;
+    my ($cv_connection, $fh, $handshake, $handshake_code) = @_;
     my $handle = AnyEvent::Handle->new(fh => $fh, on_error => _create_on_error($cv_connection));
     my $read_cb = sub {
         ## We don't receive handle object as an argument here. $handle
@@ -48,9 +54,15 @@ sub _do_handshake {
             if($handshake->version ne "draft-ietf-hybi-17") {
                 die "handshake error: unsupported WebSocket protocol version " . $handshake->version . "\n";
             }
-            my @validator_result = $validator->($handshake->req);
-            $handle->push_write($handshake->to_string);
-            $cv_connection->send(AnyEvent::WebSocket::Connection->new(handle => $handle), @validator_result);
+            my ($res, @other_results) = $handshake_code->($handshake->req, $handshake->res);
+            if(!defined($res)) {
+                croak "handshake response was undef";
+            }
+            if(ref($res) eq "Protocol::WebSocket::Response") {
+                $res = $res->to_string;
+            }
+            $handle->push_write("$res");
+            $cv_connection->send(AnyEvent::WebSocket::Connection->new(handle => $handle), @other_results);
             undef $handle;
             undef $cv_connection;
         }catch {
@@ -73,7 +85,7 @@ sub establish {
         return $cv_connection;
     }
     my $handshake = Protocol::WebSocket::Handshake::Server->new;
-    _do_handshake($cv_connection, $fh, $handshake, $self->{validator});
+    _do_handshake($cv_connection, $fh, $handshake, $self->{handshake});
     return $cv_connection;
 }
 
@@ -90,7 +102,7 @@ sub establish_psgi {
         return $cv_connection;
     }
     my $handshake = Protocol::WebSocket::Handshake::Server->new_from_psgi($env);
-    _do_handshake($cv_connection, $fh, $handshake, $self->{validator});
+    _do_handshake($cv_connection, $fh, $handshake, $self->{handshake});
     return $cv_connection;
 }
 
@@ -158,14 +170,47 @@ Fields in C<%args> are:
 
 =over
 
+=item C<handshake> => CODE (optional)
+
+A subroutine reference to customize the WebSocket handshake process.
+You can use this option to validate and preprocess the handshake request and customize the handshake response.
+
+For each request, the handshake code is called like
+
+    ($response, @other_results) = $handshake->($request, $default_response)
+
+where C<$request> is a L<Protocol::WebSocket::Request> object,
+and C<$default_response> is a L<Protocol::WebSocket::Response> object.
+The C<$handshake> code must return C<$response>. C<@other_results> are optional.
+
+The return value C<$response> is the handshake response returned to the client.
+It must be either a L<Protocol::WebSocket::Response> object,
+or a string of a valid HTTP response (including the Status-Line, the Headers and the Body).
+
+The argument C<$default_response> is a L<Protocol::WebSocket::Response> valid for the given C<$request>.
+If you don't need to manipulate the response, just return C<$default_response>. That is,
+
+    handshake => sub { $_[1] }
+
+is the minimal valid code for C<handshake>.
+
+In addition to C<$response>, you can return C<@other_results> if you want.
+Those C<@other_results> can be obtained later from the condition variable of C<establish()> method.
+
+If you throw an exception from C<$handshake> code, we think you reject the C<$request>.
+In this case, the condition variable of C<establish()> method croaks.
+
+
 =item C<validator> => CODE (optional)
 
+B<< This option is only for backward compatibility. Use C<handshake> option instead. If C<handshake> option is specified, this option is ignored. >>
+
 A subroutine reference to validate the incoming WebSocket request.
 If omitted, it accepts the request.
 
 The validator is called like
 
-    @validator_result = $validator->($request)
+    @other_results = $validator->($request)
 
 where C<$request> is a L<Protocol::WebSocket::Request> object.
 
@@ -187,11 +232,11 @@ C<$fh> is a filehandle for a connection socket, which is usually obtained by C<t
 
 Return value C<$conn_cv> is an L<AnyEvent> condition variable.
 
-In success, C<< $conn_cv->recv >> returns an L<AnyEvent::WebSocket::Connection> object and additional values returned by the validator.
-In failure (e.g. the client sent a totally invalid request or your validator threw an exception),
+In success, C<< $conn_cv->recv >> returns an L<AnyEvent::WebSocket::Connection> object and C<@other_results> returned by the handshake process.
+In failure (e.g. the client sent a totally invalid request or your handshake process threw an exception),
 C<$conn_cv> will croak an error message.
 
-    ($connection, @validator_result) = eval { $conn_cv->recv };
+    ($connection, @other_results) = eval { $conn_cv->recv };
     
     ## or in scalar context, it returns $connection only.
     $connection = eval { $conn_cv->recv };
@@ -218,7 +263,7 @@ If C<$fh> is omitted, C<< $psgi_env->{"psgix.io"} >> is used for the connection
 
 =head1 EXAMPLES
 
-=head2 Validator option
+=head2 handshake option
 
 The following server accepts WebSocket URLs such as C<ws://localhost:8080/2013/10>.
 
@@ -226,16 +271,22 @@ The following server accepts WebSocket URLs such as C<ws://localhost:8080/2013/1
     use AnyEvent::WebSocket::Server;
     
     my $server = AnyEvent::WebSocket::Server->new(
-        validator => sub {
-            my ($req) = @_;  ## Protocol::WebSocket::Request
-            
+        handshake => sub {
+            my ($req, $res) = @_;
+            ## $req is a Protocol::WebSocket::Request
+            ## $res is a Protocol::WebSocket::Response
+    
+            ## validating and parsing request.
             my $path = $req->resource_name;
             die "Invalid format" if $path !~ m{^/(\d{4})/(\d{2})};
             
             my ($year, $month) = ($1, $2);
             die "Invalid month" if $month <= 0 || $month > 12;
     
-            return ($year, $month);
+            ## setting WebSocket subprotocol in response
+            $res->subprotocol("mytest");
+            
+            return ($res, $year, $month);
         }
     );
     
@@ -276,6 +327,10 @@ Stand-alone WebSocket server and client implementation using L<IO::Async>
 
 Toshio Ito, C<< <toshioito at cpan.org> >>
 
+=head1 CONTRIBUTORS
+
+mephinet (Philipp Gortan)
+
 =head1 REPOSITORY
 
 L<https://github.com/debug-ito/AnyEvent-WebSocket-Server>
